
import BrandDataAccess from "dataAccess/brandDataAccess";
import { Dispatch } from "redux";

const typesPrefix = "BRAND_";
const types = {
   brandLoadingStart: typesPrefix + "BRAND_LOADING_START",
   brandLoadingComplete: typesPrefix + "BRAND_LOADING_COMPLETE",
   brandLoadingError: typesPrefix + "BRAND_LOADING_ERROR",

   brandProductsLoadingStart: typesPrefix + "BRAND_PRODUCT_LOADING_START",
   brandProductsLoadingComplete: typesPrefix + "BRAND_PRODUCT_LOADING_COMPLETE",
   brandProductsLoadingError: typesPrefix + "BRAND_PRODUCT_LOADING_ERROR",
};

export const actions = {

//    clearProductList: () => (dispatch: Dispatch) => {
//       dispatch({ type: types.clearProducts });
//    },

    getAllBrands: (perPage = 4, page=1) => (dispatch: Dispatch) => {
        dispatch({ type: types.brandLoadingStart, payload: page });
        BrandDataAccess.getAllBrandsWithPagination(perPage, page)
            .then((data) => {
                dispatch({ type: types.brandLoadingComplete, payload: data });
            })
            .catch((error: Error) => {
                dispatch({ type: types.brandLoadingError, payload: error.message });
            });
    },

    getProductsByBrandName: (brandName:string, perPage=12, page=1) => (dispatch: Dispatch) => {
        dispatch({ type: types.brandProductsLoadingStart });
        BrandDataAccess.getProductsByBrandName(brandName, perPage, page)
            .then((data) => {
                dispatch({ type: types.brandProductsLoadingComplete, payload: data });
            })
            .catch((error: Error) => {
                dispatch({ type: types.brandProductsLoadingError, payload: error.message });
            });
    },

};

const initialState = {
   brands: [],
   brandsInfo: {},
   brandsLoading: true,
   brandsError: false,
   brandsErrorMessage: "",

   brandProducts: [],
   brandProductsInfo: {},
   brandProductsLoading: true,
   brandProductsError: false,
   brandProductsErrorMessage: "",
};

interface Action {
   type: string;
   payload: any;
}
export const reducer = (state: any = initialState, action: Action) => {
    switch (action.type) {

        // Brands
        case types.brandLoadingStart:
            return {
                ...state,
                brandsLoading: true,
                brandsError: false ,
                brands: action.payload === 1 ? [] : state.brands,
            };

        case types.brandLoadingComplete:
            return { 
                ...state,
                brandsLoading: false,
                brandsError: false,
                brands: [ ...state.brands, ...action.payload.data ],
                brandsInfo: action.payload,
            };

        case types.brandLoadingError:
            return {
                ...state,
                brandsLoading: false,
                brandsError: true,
                brandsErrorMessage: action.payload,
            };



        // Brands products
        case types.brandProductsLoadingStart:
            return {
                ...state,
                brandProductsLoading: true,
                brandProductsError: false 
            };

        case types.brandProductsLoadingComplete:
            return { 
                ...state,
                brandProductsLoading: false,
                brandProductsError: false,
                brandProducts: [ ...state.brandProducts, ...action.payload.products.data ],
                brandProductsInfo: action.payload.products,
            };

        case types.brandProductsLoadingError:
            return {
                ...state,
                brandProductsLoading: false,
                brandProductsError: true,
                brandProductsErrorMessage: action.payload,
            };

        default:
            return state;
    }
};
