

import categoryDataAccess from "dataAccess/categoryDataAccess";
import { Dispatch } from "redux";

const typePrefix = "CATEGORY_";

const Types = {
   categoryLoadingStart: typePrefix + "PRODUCTS_LOADING_START",
   categoryLoadingComplete: typePrefix + "PRODUCTS_LOADING_COMPLETE",
   categoryLoadingError: typePrefix + "PRODUCTS_LOADING_ERROR",

   categoryTreeLoadingStart: typePrefix + "CATEGORY_TREE_LOADING_START",
   categoryTreeLoadingComplete: typePrefix + "CATEGORY_TREE_LOADING_COMPLETE",
   categoryTreeLoadingError: typePrefix + "CATEGORY_TREE_LOADING_ERROR",

   allCategoryLoadingStart: typePrefix + "ALL_CATEGORY_LOADING_START",
   allCategoryLoadingComplete: typePrefix + "ALL_CATEGORY_LOADING_COMPLETE",
   allCategoryLoadingError: typePrefix + "ALL_CATEGORY_LOADING_ERROR",

   allSubCategoryLoadingStart: typePrefix + "ALL_SUB_CATEGORY_LOADING_START",
   allSubCategoryLoadingComplete: typePrefix + "ALL_SUB_CATEGORY_LOADING_COMPLETE",
   allSubCategoryLoadingError: typePrefix + "ALL_SUB_CATEGORY_LOADING_ERROR",
};

export const actions = {

   getCategory: (categoryId) => (dispatch: Dispatch) => {
      dispatch({ type: Types.categoryLoadingStart });
      categoryDataAccess
         .getCategory(categoryId)
         .then((category) => {
            dispatch({ type: Types.categoryLoadingComplete, payload: category });
         })
         .catch((error) => {
            dispatch({ type: Types.categoryLoadingError, payload: error.message });
         });
   },

   getCategoryTree: () => (dispatch: Dispatch)=>{
      dispatch({ type: Types.categoryTreeLoadingStart });
      categoryDataAccess.getCategoryTreeView()
         .then((data) => {
            data.sort((a, b) => {
               return b.child.length - a.child.length;
            });
            dispatch({ type: Types.categoryTreeLoadingComplete, payload: data });
         })
         .catch((error) => {
            dispatch({ type: Types.categoryTreeLoadingError, payload: error.message });
         });
   },

   getAllCategories: (page=1) => (dispatch: Dispatch)=>{
      dispatch({ type: Types.allCategoryLoadingStart, payload: page });
      categoryDataAccess.getAllCategories(12, page)
         .then((data) => {
            dispatch({ type: Types.allCategoryLoadingComplete, payload: data });
         })
         .catch((error) => {
            dispatch({ type: Types.allCategoryLoadingError, payload: error.message });
         });
   },

   getAllSubCategories: (categorySlug, page=1) => (dispatch: Dispatch)=>{
      dispatch({ type: Types.allSubCategoryLoadingStart , payload: page});
      categoryDataAccess.getAllSubCategories(categorySlug, 12, page)
         .then((data) => {
            dispatch({ type: Types.allSubCategoryLoadingComplete, payload: data });
         })
         .catch((error) => {
            dispatch({ type: Types.allSubCategoryLoadingError, payload: error.message });
         });
   },


};

const initialState = {
   category: {},
   categoryLoading: false,
   categoryError: false,
   categoryErrorMessage: "",

   categoryTree: [],
   categoryTreeLoading: false,
   categoryTreeError: false,
   categoryTreeErrorMessage: "",

   allCategory: [],
   allCategoryInfo: {},
   allCategoryLoading: false,
   allCategoryError: false,
   allCategoryErrorMessage: "",

   allSubCategory: [],
   allSubCategoryInfo: {},
   allSubCategoryLoading: false,
   allSubCategoryError: false,
   allSubCategoryErrorMessage: "",
   parentCategory: {},
};

export const reducer = (state: any = initialState, action: any) => {
   switch (action.type) {

      case Types.categoryLoadingStart:
         return { ...state, categoryLoading: true, categoryError: false };

      case Types.categoryLoadingComplete:
         return { ...state, categoryLoading: false, categoryError: false, category: action.payload };

      case Types.categoryLoadingError:
         return { ...state, categoryLoading: false, categoryError: true, categoryErrorMessage: action.payload };
      
      // category tree
      case Types.categoryTreeLoadingStart:
         return { 
            ...state,
            categoryTreeLoading: true,
            categoryTreeError: false 
         };
      case Types.categoryTreeLoadingComplete:
         return { 
            ...state,
            categoryTreeLoading: false,
            categoryTreeError: false,
            categoryTree: [...state.categoryTree, ...action.payload],
         };
      case Types.categoryTreeLoadingError:
         return { 
            ...state,
            categoryTreeLoading: false,
            categoryTreeError: true,
            categoryTreeErrorMessage: action.payload,
         };

      // all category
      case Types.allCategoryLoadingStart:
         return { 
            ...state,
            allCategoryLoading: true,
            allCategoryError: false,
            allCategory: action.payload === 1 ? [] : state.allCategory,
         };
      case Types.allCategoryLoadingComplete:
         return { 
            ...state,
            allCategoryLoading: false,
            allCategoryError: false,
            allCategory: [...state.allCategory, ...action.payload.data],
            allCategoryInfo: action.payload,
         };
      case Types.allCategoryLoadingError:
         return { 
            ...state,
            allCategoryLoading: false,
            allCategoryError: true,
            allCategoryErrorMessage: action.payload,
         };



      // all sub categories
      case Types.allSubCategoryLoadingStart:
         return {
            ...state,
            allSubCategoryLoading: true,
            allSubCategoryError: false ,
            allSubCategory: action.payload === 1 ? [] : state.allSubCategory,
         };
      case Types.allSubCategoryLoadingComplete:
         return { 
            ...state,
            allSubCategoryLoading: false,
            allSubCategoryError: false,
            allSubCategory: [...state.allSubCategory, ...action.payload.subcategories.data],
            allSubCategoryInfo: action.payload.subcategories,
            parentCategory: action.payload.category,
         };
      case Types.allSubCategoryLoadingError:
         return { 
            ...state,
            allSubCategoryLoading: false,
            allSubCategoryError: true,
            allSubCategoryErrorMessage: action.payload,
         };

      default:
         return state;
   }
};
