import LanguageText from "components/common/languageText";
import React, { useEffect, useState } from "react";
import { Card, Col, Row } from "react-bootstrap";
import {
  CarouselProvider,
  Slider,
  Slide,
  ButtonBack,
  ButtonNext,
} from "pure-react-carousel";

//import "components/ourOffers/node_modules/pure-react-carousel/dist/react-carousel.es.css";

import ProductDataAccess from "dataAccess/productDataAccess";
import { Link } from "react-router-dom";

const ExploreCategories = (props: any) => {
  const [windowWidth, setWindowWidth] = useState(window.innerWidth);
  const [showCategoryIndex, setShowCategoryIndex] = useState(-1);
  const [seeAllBtn, setSeeAllBtn] = useState(false);
  const [allCategory, setAllCategory] = useState([]);
  const [totalProduct, setTotalProduct] = useState();

  useEffect(() => {
    const handleResize = () => {
      setWindowWidth(window.innerWidth);
    };
    window.addEventListener("resize", handleResize);
    return () => window.removeEventListener("resize", handleResize);
  }, []);

  const getVisibleSlides = () => {
    if (windowWidth > 992) {
      return 5;
    } else if (windowWidth > 768) {
      return 4;
    } else if (windowWidth > 576) {
      return 2;
    } else {
      return 1;
    }
  };
  useEffect(() => {
    ProductDataAccess.getAllCategory()
      .then((data) => {
        setAllCategory(data);
        setTotalProduct(data.length);
      })
      .catch((error) => {});
  }, []);

  return (
    <div className="explore-categories">
      <CarouselProvider
        naturalSlideWidth={500}
        naturalSlideHeight={140}
        totalSlides={totalProduct}
        visibleSlides={getVisibleSlides()}
        isIntrinsicHeight={true}
        className="products-slider"
        step={2}
      >
        <Row>
          <Col className="products-row" lg={6} md={6} sm={12} xs={12}>
            <p className="explore-categories-header">
              {" "}
              <LanguageText
                firstKey="page"
                secondKey="explore_categories"
              />{" "}
            </p>
          </Col>
          <Col lg={6} md={6} sm={12} xs={12}>
            <div className="slider explore-categories-header">
            <Link to="/view-all-category">
                {" "}
                <span className="show-all" 
                style={{fontSize:"14px",fontWeight:400,marginRight: "16px",borderBottom: "1px solid #787c87",color: "#787c87" }}> Show All </span>{" "}
              </Link>
              <ButtonBack style={{ background: "transparent", border: "0px" }}>
                <img src="/images/arrowleft.png" alt="Previous product" />
              </ButtonBack>
              &nbsp;
              <ButtonNext style={{ background: "transparent", border: "0px" }}>
                <img src="/images/arrowright.png" alt="Next product" />
              </ButtonNext>
              &nbsp;
            </div>
          </Col>
        </Row>

        <Slider>
          {allCategory !== undefined &&
            allCategory.map((category, index) => (
              <Slide index={index}>
                <div
                  onMouseEnter={() => {
                    setShowCategoryIndex(index);
                  }}
                  onMouseLeave={() => setShowCategoryIndex(-1)}
                >
                  {showCategoryIndex === index ? (
                    <Card className="explore-slider">
                      <div className="slider-image">
                        <img
                          src={category.image ?? '/images/health.png'}
                          className="target-image"
                          alt="category"
                        />
                      </div>
                      <div
                        className="button"
                        onMouseEnter={() => {
                          setSeeAllBtn(true);
                        }}
                        onMouseLeave={() => setSeeAllBtn(false)}
                      >
                        {seeAllBtn ? (
                          // <Link to={"/our-products/" + category.slug}>
                          <Link to={"/category/subcategory/" + category.slug}>
                            <button className="see-all-btn">
                              {" "}
                              See All &nbsp; &nbsp;{" "}
                              <img src="/images/arrow-right-white.png" />
                            </button>
                          </Link>
                        ) : (
                          // <Link to={"/our-products/" + category.slug}>
                          <Link to={"/category/subcategory/" + category.slug}>
                            <button className="see-all-btn">
                              {" "}
                              See All &nbsp; &nbsp;{" "}
                              <img src="/images/red-arrow-right.png" />
                            </button>
                          </Link>
                        )}
                      </div>
                    </Card>
                  ) : (
                    <Card className="explore-slider">
                      <img
                          src={category.image ?? '/images/health.png'}
                        className="card-category-image"
                        alt="category"
                      />
                      <Card.Body>
                        <Card.Title className="card-title">
                          {" "}
                          {category.name}{" "}
                        </Card.Title>
                        <Card.Text className="card-text">
                          {/* We have two sub-categories like{" "}
                        <strong>fruits, vegetables.</strong> */}
                        </Card.Text>
                      </Card.Body>
                    </Card>
                  )}
                </div>
              </Slide>
            ))}
        </Slider>
      </CarouselProvider>
    </div>
  );
};
export default ExploreCategories;
