import React, {useEffect, useState} from "react";
import {connect, useDispatch} from "react-redux";
import {Col, Row, Button, Spinner} from "react-bootstrap";
import {Link} from "react-router-dom";
import OfferDataAccess from "dataAccess/offerDataAccess";
import {actions as cartActions} from "@redux/cartRedux";
import {actions as favoriteActions} from "@redux/favoriteRedux";
import cartServices from "services/cartServices";
import favoriteServices from "services/favouriteServices";
import {isUserLoggedIn} from "services/userServices";
import {FontAwesomeIcon} from "@fortawesome/react-fontawesome";
import Rating from "react-rating";
import SuperSaleProducts from "components/ourOffers/superSale";
import ProductDataAccess from "dataAccess/productDataAccess";
import {toast} from "react-toastify";
import {Helmet} from "react-helmet";
import SliderDataAccess from "dataAccess/sliderDataAccess";
import NewArrivalDataAccess from "dataAccess/newArrivalDataAccess";
import {actions as newArrivalProductsActions} from "@redux/newArrivalRedux";
import {actions as productActions} from "@redux/productRedux";
import SpinnerLocal from "components/spinner";
import {Range} from 'rc-slider';
import 'rc-slider/assets/index.css';

const SubCategoryProducts = (props: any) => {
    const dispatch = useDispatch();
    const categorySlug2 = props.match.params.categorySlug2;
    const [filters, setFilters] = useState({
        brands: undefined,
        price: undefined,
    });

    const [filterValues, setFilterValues] = useState({
        filter: false,
        brand: '',
        category: '',
        min: 0,
        max: 1000000000,
    })

    const [currency, setCurrency] = useState(null);
    useEffect(() => {
        SliderDataAccess.getSettingData()
            .then((data) => {
                setCurrency(data.currency_symbol);
            })
            .catch((error) => {
            });
    }, []);

    useEffect(() => {
        setFilters({
            brands: undefined,
            price: undefined,
        })
        dispatch(productActions.getProductsByCategorySlug(categorySlug2, 12));
        ProductDataAccess.getProductFilters(categorySlug2).then((response => {
            if (response.error === false) {
                setFilters(response?.data)
                setFilterValues({
                    ...filterValues,
                    filter: false,
                    min: parseInt(response?.data?.price?.min),
                    max: parseInt(response?.data?.price?.max),
                })
            }
        }))

    }, [dispatch, categorySlug2]);


    const handleFilter = () => {
        dispatch(productActions.getProductsByCategorySlug(categorySlug2, 12, 1, true, filterValues.brand, filterValues.min, filterValues.max));
        setFilterValues({
            ...filterValues,
            filter: true,
        })
    }

    const handleClear = () => {
        setFilterValues({
            filter: false,
            brand: '',
            category: '',
            min: filters!.price?.min ? parseInt(filters!.price?.min) : 0,
            max: filters!.price?.max ? parseInt(filters!.price?.max) : 100000000,
        })
        dispatch(productActions.getProductsByCategorySlug(categorySlug2, 12));
    }

    const [price, setPrice] = useState<any>(100);
    const requestToStock = (productId) => {
        ProductDataAccess.ProductRequestToStock(productId)
            .then((data) => {
                toast.success(data);
            })
            .catch((error) => {
                toast.error("Login first and send request again");
            });
    };

    // if (
    //     props.product.categoryProductsLoading &&
    //     props.product.categoryProducts.length === 0
    // ) {
    //     return (
    //         <Row>
    //             <SpinnerLocal height="75vh"/>
    //         </Row>
    //     );
    // }

    return (
        <div>
            <Helmet>
                <title> Genx Products </title>
                <meta name="description" content="Shopstick Products All"/>
            </Helmet>
            <div className="regular-offer">
                <div className="container-fluid">
                    <div className="row">
                        <div className="col-lg-3 col-md-3">
                            <h3 className="category-top-title"> {categorySlug2.split('-').map(slag => <span
                                className="text-capitalize">{slag} </span>)}</h3>
                        </div>
                    </div>
                    <div className="row">
                        {filters.brands ? (
                            <div className="col-lg-3 col-md-3">
                                <div className="card" style={{padding: "0px 25px 20px 25px"}}>
                                    <div className="shop_sidebar_wrapper">
                                        {/*<div className="shop_Search">*/}
                                        {/*  <form>*/}
                                        {/*    <input*/}
                                        {/*      type="text"*/}
                                        {/*      className="form-control"*/}
                                        {/*      placeholder="Search..."*/}
                                        {/*      onKeyUp={() => {}}*/}
                                        {/*    />*/}
                                        {/*    <button>*/}
                                        {/*      <img*/}
                                        {/*        src="https://andshop-react.netlify.app/static/media/search.4672f25b.svg"*/}
                                        {/*        alt="img"*/}
                                        {/*      />*/}
                                        {/*    </button>*/}
                                        {/*  </form>*/}
                                        {/*</div>*/}
                                        <div className="">
                                            <h4 className="pb-2">Product Brands</h4>
                                            <div className="position-relative">
                                                <form className="overflow-auto slim-thumb" style={{maxHeight: 300}}>
                                                    {filters!.brands?.map(brand => (
                                                        <label className="custom_boxed">
                                                            {brand.name}
                                                            <input type="radio" name="radio" checked={filterValues.brand === brand.name} onClick={() => {
                                                                setFilterValues({
                                                                    ...filterValues,
                                                                    brand: brand.name,
                                                                })
                                                            }}/>
                                                            <span className="checkmark"></span>
                                                        </label>
                                                    ))}
                                                </form>
                                            </div>

                                        </div>
                                        <div className="shop_sidebar_boxed">
                                            <h4>Price</h4>
                                            <div className="price_filter">
                                                <Range
                                                    onChange={(values) => {
                                                        setFilterValues({
                                                            ...filterValues,
                                                            min: values[0],
                                                            max: values[1],
                                                        })
                                                    }}
                                                    value={[filterValues.min, filterValues.max]}
                                                    min={filters!.price?.min ? parseInt(filters!.price?.min) : 0}
                                                    max={filters!.price?.max ? parseInt(filters!.price?.max) : 100000000}
                                                    railStyle={{
                                                        height: 10
                                                    }}
                                                    trackStyle={[{
                                                        backgroundColor: '#007bff',
                                                        height: 10
                                                    }]}
                                                    handleStyle={[{
                                                        borderColor: '#007bff',
                                                        padding: 8,
                                                    }, {
                                                        borderColor: '#007bff',
                                                        padding: 8,
                                                    }]}

                                                />
                                                <div className="price_slider_amount mt-3 d-flex justify-content-between">
                                                    <span>Min : ${filterValues.min}</span>
                                                    <span>Max : ${filterValues.max}</span>
                                                </div>
                                            </div>
                                        </div>
                                        {/*<div className="shop_sidebar_boxed">*/}
                                        {/*    <h4>Color</h4>*/}
                                        {/*    <div className="product-variable-color">*/}
                                        {/*        <label htmlFor="modal-product-color-red6">*/}
                                        {/*            <input*/}
                                        {/*                name="modal-product-color"*/}
                                        {/*                id="modal-product-color-red6"*/}
                                        {/*                className="color-select"*/}
                                        {/*                type="radio"*/}
                                        {/*            />*/}
                                        {/*            <span className="product-color-red"></span>*/}
                                        {/*        </label>*/}
                                        {/*        <label htmlFor="modal-product-color-tomato1">*/}
                                        {/*            <input*/}
                                        {/*                name="modal-product-color"*/}
                                        {/*                id="modal-product-color-tomato1"*/}
                                        {/*                className="color-select"*/}
                                        {/*                type="radio"*/}
                                        {/*            />*/}
                                        {/*            <span className="product-color-tomato"></span>*/}
                                        {/*        </label>*/}
                                        {/*        <label htmlFor="modal-product-color-green2">*/}
                                        {/*            <input*/}
                                        {/*                name="modal-product-color"*/}
                                        {/*                id="modal-product-color-green2"*/}
                                        {/*                className="color-select"*/}
                                        {/*                type="radio"*/}
                                        {/*                defaultChecked*/}
                                        {/*            />*/}
                                        {/*            <span className="product-color-green"></span>*/}
                                        {/*        </label>*/}
                                        {/*        <label htmlFor="modal-product-color-light-green3">*/}
                                        {/*            <input*/}
                                        {/*                name="modal-product-color"*/}
                                        {/*                id="modal-product-color-light-green3"*/}
                                        {/*                className="color-select"*/}
                                        {/*                type="radio"*/}
                                        {/*            />*/}
                                        {/*            <span className="product-color-light-green"></span>*/}
                                        {/*        </label>*/}
                                        {/*        <label htmlFor="modal-product-color-blue4">*/}
                                        {/*            <input*/}
                                        {/*                name="modal-product-color"*/}
                                        {/*                id="modal-product-color-blue4"*/}
                                        {/*                className="color-select"*/}
                                        {/*                type="radio"*/}
                                        {/*            />*/}
                                        {/*            <span className="product-color-blue"></span>*/}
                                        {/*        </label>*/}
                                        {/*        <label htmlFor="modal-product-color-light-blue5">*/}
                                        {/*            <input*/}
                                        {/*                name="modal-product-color"*/}
                                        {/*                id="modal-product-color-light-blue5"*/}
                                        {/*                className="color-select"*/}
                                        {/*                type="radio"*/}
                                        {/*            />*/}
                                        {/*            <span className="product-color-light-blue"></span>*/}
                                        {/*        </label>*/}
                                        {/*    </div>*/}
                                        {/*</div>*/}
                                        {/*<div className="shop_sidebar_boxed">*/}
                                        {/*    <h4>Size</h4>*/}
                                        {/*    <form id="sizes_input">*/}
                                        {/*        <label className="custom_boxed">*/}
                                        {/*            XS*/}
                                        {/*            <input type="radio" name="radio"/>*/}
                                        {/*            <span className="checkmark"></span>*/}
                                        {/*        </label>*/}
                                        {/*        <label className="custom_boxed">*/}
                                        {/*            S*/}
                                        {/*            <input type="radio" name="radio"/>*/}
                                        {/*            <span className="checkmark"></span>*/}
                                        {/*        </label>*/}
                                        {/*        <label className="custom_boxed">*/}
                                        {/*            M*/}
                                        {/*            <input type="radio" name="radio" defaultChecked/>*/}
                                        {/*            <span className="checkmark"></span>*/}
                                        {/*        </label>*/}
                                        {/*        <label className="custom_boxed">*/}
                                        {/*            L*/}
                                        {/*            <input type="radio" name="radio"/>*/}
                                        {/*            <span className="checkmark"></span>*/}
                                        {/*        </label>*/}
                                        {/*        <label className="custom_boxed">*/}
                                        {/*            XL*/}
                                        {/*            <input type="radio" name="radio"/>*/}
                                        {/*            <span className="checkmark"></span>*/}
                                        {/*        </label>*/}
                                        {/*    </form>*/}
                                        {/*</div>*/}
                                        <div className="mt-3 d-flex justify-content-end">
                                            <button className="btn btn-secondary mr-2" onClick={handleClear}>Reset</button>
                                            <button className="btn btn-primary" onClick={handleFilter}>Search</button>
                                        </div>
                                    </div>
                                </div>
                            </div>
                        ) : (
                            <div className="col-lg-1 col-md-1"/>
                        )}

                        <div className="col-lg-9 col-md-9">
                            <div className="row">
                                {props.product.categoryProductsLoading && props.product.categoryProducts.length === 0 ? (
                                    <SpinnerLocal height="55vh"/>
                                ) : (
                                    <>
                                        {props.product.categoryProducts.length > 0 &&
                                        props.product.categoryProducts.map((product, index) => (
                                            <div className="col-lg-4 col-md-6 col-sm-12" key={index}>
                                                <div className="new-card-design">
                                                    <div className="card">
                                                        <div className="stock-new-design">
                                                            {product.is_available == 1 ? (
                                                                <div>
                                                                    {product.is_visible_stock == 1 ? (
                                                                        <h6 className="stock-text">
                                                                            In Stock
                                                                            <span className="">
                                      ({product.quantity})
                                    </span>
                                                                        </h6>
                                                                    ) : (
                                                                        <h6 className="stock-text2">In Stock</h6>
                                                                    )}
                                                                </div>
                                                            ) : (
                                                                <h6 className="stock-text3">Out Of Stock</h6>
                                                            )}
                                                        </div>
                                                        <Link
                                                            to={"/product-details/" + product.slug}
                                                            className="product-image-big"
                                                        >
                                                            <img
                                                                className="card-img-top"
                                                                src={product.image1}
                                                                alt="Cardimagecap"
                                                            />
                                                        </Link>
                                                        <div className="card-body">
                                                            <div className="d-flex">
                                                                <div>
                                                                    <div style={{height: "30px"}}>
                                  <span>
                                    {product.name.length > 20 ? (
                                        <p className="card-product-name">
                                            {product.name.substring(0, 15) + "..."}
                                        </p>
                                    ) : (
                                        <p className="card-product-name">
                                            {product.name}
                                        </p>
                                    )}
                                  </span>
                                                                    </div>
                                                                    <div
                                                                        className="rating-content"
                                                                        style={{color: "gold"}}
                                                                    >
                                                                        {product.rating.avg_rating !== 0 && (
                                                                            <span
                                                                                style={{
                                                                                    marginRight: "3px",
                                                                                    fontSize: "12px",
                                                                                }}
                                                                            >
                                      <Rating
                                          fullSymbol="fa fa-star"
                                          emptySymbol="fa fa-star-o"
                                          readonly
                                          initialRating={
                                              product.rating.avg_rating
                                          }
                                      />
                                      <span style={{color: "red"}}>
                                        ({product.rating.rating_number})
                                      </span>
                                    </span>
                                                                        )}
                                                                        {product.rating.avg_rating === 0 && (
                                                                            <div
                                                                                style={{
                                                                                    color: "#FFD700",
                                                                                    marginTop: "9px",
                                                                                }}
                                                                            >
                                                                                <div
                                                                                    className="d-flex"
                                                                                    style={{
                                                                                        fontSize: "11px",
                                                                                    }}
                                                                                >
                                                                                    <i className="far fa-star mr-1"></i>
                                                                                    <i className="far fa-star mr-1"></i>
                                                                                    <i className="far fa-star mr-1"></i>
                                                                                    <i className="far fa-star mr-1"></i>
                                                                                    <i className="far fa-star mr-1"></i>
                                                                                </div>
                                                                            </div>
                                                                        )}
                                                                    </div>
                                                                    {/* <div>
                                <div>
                                  <p className="text-secondary">
                                    {product.unit_quantity} {product.unit.name}
                                  </p>
                                </div>
                              </div> */}

                                                                    <span className="price-and-cart-btn">
                                  <p className="product-price">
                                    {currency}{" "}
                                      {product.discount_price == 0
                                          ? product.price
                                          : product.discount_price}{" "}
                                      {product.discount_price != 0 && (
                                          <del className="product-before-price">
                                              {currency} {product.price}
                                          </del>
                                      )}
                                  </p>
                                </span>
                                                                </div>
                                                                <div className="ml-auto">
                                                                    <div>
                                                                        {favoriteServices.isProductOnFavorite(
                                                                            product.id,
                                                                            props.favorite.favorite
                                                                        ) ? (
                                                                            <span
                                                                                onClick={() => {
                                                                                    dispatch(
                                                                                        favoriteActions.removeProductFromFavorite(
                                                                                            product.id,
                                                                                            isUserLoggedIn(props.user.user)
                                                                                        )
                                                                                    );
                                                                                }}
                                                                                className="favorite-icon"
                                                                            >
                                      <img
                                          className="like"
                                          src={
                                              process.env.PUBLIC_URL +
                                              "/images/likered.png"
                                          }
                                          alt=""
                                      />
                                    </span>
                                                                        ) : (
                                                                            <span
                                                                                onClick={() => {
                                                                                    dispatch(
                                                                                        favoriteActions.addProductToFavorite(
                                                                                            product.id,
                                                                                            isUserLoggedIn(props.user.user)
                                                                                        )
                                                                                    );
                                                                                }}
                                                                                className="favorite-icon"
                                                                            >
                                      <img
                                          className="like"
                                          src={
                                              process.env.PUBLIC_URL +
                                              "/images/like.png"
                                          }
                                          alt=""
                                      />
                                    </span>
                                                                        )}
                                                                    </div>
                                                                    <div>
                                                                        {product.is_available !== 0 ? (
                                                                            <Link
                                                                                to={
                                                                                    "/compair-product-detail/" +
                                                                                    product.slug
                                                                                }
                                                                            >
                                                                                <h6
                                                                                    className="compair-text"
                                                                                    style={{
                                                                                        // width: "70px",
                                                                                        // padding: "2px 2px",
                                                                                        color: "#40BFFF",
                                                                                        // borderColor: "#40BFFF",
                                                                                        fontSize: "14px",
                                                                                    }}
                                                                                >
                                                                                    Compare
                                                                                </h6>
                                                                            </Link>
                                                                        ) : (
                                                                            ""
                                                                        )}
                                                                    </div>
                                                                    <div className="row justify-content-between">
                                                                        <div className="col-auto">
                                                                            {product.is_available === 0 ? (
                                                                                <div className="pb-3">
                                                                                    <p
                                                                                        className="cart-button"
                                                                                        style={{
                                                                                            width: "90px",
                                                                                            padding: "0px 2px 3px 2px",
                                                                                        }}
                                                                                        onClick={() => {
                                                                                            requestToStock(product.id);
                                                                                        }}
                                                                                    >
                                          <span
                                              className="request-to-stock-btn-text"
                                              style={{fontSize: "11px"}}
                                          >
                                            Notify me
                                          </span>
                                                                                    </p>
                                                                                    <p
                                                                                        className="out-of-stock"
                                                                                        style={{
                                                                                            fontSize: "13px",
                                                                                        }}
                                                                                    >
                                                                                        {" "}
                                                                                        {/* Out Of Stock{" "} */}
                                                                                    </p>
                                                                                </div>
                                                                            ) : cartServices.isProductOnCart(
                                                                                product.id,
                                                                                props.cart
                                                                            ) ? (
                                                                                // <p className="cart-btn-after-click">
                                                                                //   <span className="after-click">
                                                                                //     <span
                                                                                //       className="cart-minus-sign"
                                                                                //       onClick={() => {
                                                                                //         props.dispatch(
                                                                                //           cartActions.decreaseProductQuantity(
                                                                                //             product.id,
                                                                                //             cartServices.getCartQuantity(
                                                                                //               product.id,
                                                                                //               props.cart
                                                                                //             ),
                                                                                //             isUserLoggedIn(
                                                                                //               props.user.user
                                                                                //             )
                                                                                //           )
                                                                                //         );
                                                                                //       }}
                                                                                //     >
                                                                                //       <FontAwesomeIcon icon="minus" />
                                                                                //     </span>
                                                                                //     &nbsp;
                                                                                //     <span>
                                                                                //       {cartServices.getCartQuantity(
                                                                                //         product.id,
                                                                                //         props.cart
                                                                                //       )}
                                                                                //     </span>
                                                                                //     &nbsp;
                                                                                //     <span
                                                                                //       className="cart-plus-sign"
                                                                                //       onClick={() => {
                                                                                //         cartServices.getCartQuantity(
                                                                                //           product.id,
                                                                                //           props.cart
                                                                                //         ) === product.quantity
                                                                                //           ? toast.error("stock out")
                                                                                //           : dispatch(
                                                                                //               cartActions.increaseProductQuantity(
                                                                                //                 product.id,
                                                                                //                 cartServices.getCartQuantity(
                                                                                //                   product.id,
                                                                                //                   props.cart
                                                                                //                 ),
                                                                                //                 isUserLoggedIn(
                                                                                //                   props.user.user
                                                                                //                 )
                                                                                //               )
                                                                                //             );
                                                                                //       }}
                                                                                //     >
                                                                                //       <FontAwesomeIcon icon="plus" />
                                                                                //     </span>
                                                                                //   </span>
                                                                                // </p>
                                                                                <p
                                                                                    className="cart-button"
                                                                                    style={{
                                                                                        width: "90px",
                                                                                        padding: "0px 2px 3px 2px",
                                                                                    }}
                                                                                    onClick={() => {
                                                                                        dispatch(
                                                                                            cartActions.removeFromCart(
                                                                                                product.id,
                                                                                                isUserLoggedIn(props.user.user)
                                                                                            )
                                                                                        );
                                                                                    }}
                                                                                >
                                        <span className="before-click">
                                          <span className="cart">
                                            <img
                                                className="cart-image"
                                                src={
                                                    process.env.PUBLIC_URL +
                                                    "/images/card-red.png"
                                                }
                                                alt=""
                                                style={{width: "12px"}}
                                            />
                                          </span>
                                          <span
                                              className="cart responsive-width"
                                              style={{fontSize: "11px"}}
                                          >
                                            Added
                                          </span>
                                        </span>
                                                                                </p>
                                                                            ) : (
                                                                                <p
                                                                                    className="cart-button"
                                                                                    style={{
                                                                                        width: "fit-content",
                                                                                        padding: "0px 2px 3px 2px",
                                                                                    }}
                                                                                    onClick={() => {
                                                                                        dispatch(
                                                                                            cartActions.addProductToCart(
                                                                                                product.id,
                                                                                                isUserLoggedIn(props.user.user)
                                                                                            )
                                                                                        );
                                                                                    }}
                                                                                >
                                        <span
                                            className="before-click"
                                            style={{padding: "10px"}}
                                        >
                                          <span className="cart">
                                            <img
                                                className="cart-image"
                                                src={
                                                    process.env.PUBLIC_URL +
                                                    "/images/card-red.png"
                                                }
                                                alt=""
                                                style={{width: "12px"}}
                                            />
                                          </span>
                                          <span
                                              className="cart responsive-width"
                                              style={{fontSize: "11px"}}
                                          >
                                            Add to Cart
                                          </span>
                                        </span>
                                                                                </p>
                                                                            )}
                                                                        </div>
                                                                        <div className="ml-auto">
                                                                            {/* {product.is_available !== 0 ? (
                                    <Link
                                      to={"/compair-product-detail/" + product.slug}
                                    >
                                      <h6
                                        className="mt-1"
                                        style={{
                                          // width: "70px",
                                          // padding: "2px 2px",
                                          color: "#40BFFF",
                                          // borderColor: "#40BFFF",
                                          fontSize: "14px",
                                        }}
                                      >
                                        Compair
                                      </h6>
                                    </Link>
                                  ) : (
                                    ""
                                  )} */}
                                                                        </div>
                                                                    </div>
                                                                </div>
                                                            </div>
                                                        </div>
                                                    </div>
                                                </div>
                                            </div>
                                        ))}
                                    </>
                                )}
                            </div>
                            {/* show more products loading */}
                            <div className="row">
                                {(props.product.categoryProductsLoading && props.product.categoryProducts.length > 0) && (
                                    <Row>
                                        <SpinnerLocal/>
                                    </Row>
                                )}
                            </div>

                            <div className="row">
                                {(!props.product.categoryProductsLoading && props.product.categoryProducts.length === 0) && (
                                    <div className="w-100 d-flex justify-content-center align-items-center" style={{height: "50vh", fontSize: 34}}>
                                        No products found
                                    </div>
                                )}
                            </div>

                            {/* See More */}
                            {!props.product.categoryProductsLoading &&
                            props.product.categoryProducts.length > 0 &&
                            props.product.categoryProductsInfo.last_page &&
                            props.product.categoryProductsInfo.current_page !==
                            props.product.categoryProductsInfo.last_page && (
                                <Row style={{textAlign: "center", marginTop: "20px"}}>
                                    <Col>
                                        <Button
                                            className="load-more"
                                            variant="primary"
                                            onClick={() => {
                                                dispatch(
                                                    productActions.getProductsByCategorySlug(
                                                        categorySlug2,
                                                        12,
                                                        props.product.categoryProductsInfo.current_page +
                                                        1,
                                                        filterValues.filter, filterValues.brand, filterValues.min, filterValues.max
                                                    )
                                                );
                                            }}
                                        >
                                            <span className="btn-text">Load more</span>
                                        </Button>
                                    </Col>
                                </Row>
                            )}
                        </div>
                    </div>
                </div>
            </div>
        </div>
    );
};

const mapStateToProps = ({product, cart, favorite, user}) => {
    return {product, cart: cart.cart, favorite, user};
};
export default connect(mapStateToProps)(SubCategoryProducts);
