import OutsideClickCatch from "hoc/outsideClickCatch";
import React, { useState } from "react";
import { Button, Form } from "react-bootstrap";

interface Props {
   label: string;
   values: Array<any>;
   selectedValues: Array<string>;
   setSelectedValues: Function;
}

const ShortByFilter = (props: Props) => {
   const [isExpanded, setIsExpanded] = useState(false);

   const removeSelectedItem = (item) => {
      let selectedValues = props.selectedValues.filter((_item) => _item.toString() !== item.toString());
      props.setSelectedValues(selectedValues);
   };
   const addSelectedItem = (item) => {
      let selectedValues = props.selectedValues;
      selectedValues.push(item.toString());
      props.setSelectedValues(selectedValues);
   };
   const onItemClick = (item) => {
      if (props.selectedValues.includes(item.toString())) {
         removeSelectedItem(item);
      } else {
         addSelectedItem(item);
      }
   };

   return (
      <span
         className="filter-btn-main"
         style={{ position: "relative" }}
         onClick={() => {
            if (!isExpanded) setIsExpanded(true);
         }}
      >
         <span> {props.label} </span>

         {isExpanded && (
            <OutsideClickCatch
               hideMenu={() => {
                  setIsExpanded(false);
               }}
            >
               <div className="short-by-filter">
                  {props.values.map((item, index) => {
                     const isChecked = props.selectedValues.includes(item.id.toString());
                     return (
                        <div className="filter-item" key={index}>
                           <Form.Check
                              defaultChecked={isChecked}
                              onClick={() => {
                                 onItemClick(item.id);
                              }}
                              custom
                              type={"checkbox"}
                              id={`custom-${item.id}`}
                              label={item.name}
                              className="short-by-pre-circle"
                           />
                        </div>
                     );
                  })}
               </div>
            </OutsideClickCatch>
         )}
      </span>
   );
};
export default ShortByFilter;
